import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import java.io.*;

public class GraphBox extends JDialog
{

	final private int height = 512;
	final private int width = 512;
	private int logNum = 0;

	private double xMin;
	private double xMax;
	private double xStep;
	private double yMin;
	private double yMax;
	private double yStep;
	private double totalHeight;
	private double totalWidth;
	private Frame parent;
	private Equation graph;
	

	private Point2D.Double leftXAxis = new Point2D.Double();
	private Point2D.Double rightXAxis = new Point2D.Double();
	private Point2D.Double topYAxis = new Point2D.Double();
	private Point2D.Double bottomYAxis = new Point2D.Double();
	private double pixelsForXMax;
	private double pixelsForYMax;
	private double pixelsForXMin;
	private double pixelsForYMin;

	public GraphBox(double xmin, double xmax, double xstep, double ymin, double ymax, double ystep)
	{
		setTitle("Graph");
		setResizable(false);
		setBounds(0, 0, width, height);
		addWindowListener(new DialogListener());
		xMin = xmin;
		xMax = xmax;
		xStep = xstep;
		yMin = ymin;
		yMax = ymax;
		yStep = ystep;
		calcPoints();
		repaint();
		setVisible(true);
	}

	private void calcPoints()
	{	
		if((xMin < 0) && (xMax >= 0))
		{
			totalWidth = -xMin + xMax;
			topYAxis.setLocation(((-xMin / totalWidth) * width), 0);
			bottomYAxis.setLocation(((-xMin / totalWidth) * width), height); 
		}
		else if((xMin < 0) && (xMax < 0))
		{
			totalWidth = -xMin - xMax;
			topYAxis.setLocation(((-xMin / totalWidth) * width), 0);
			bottomYAxis.setLocation(((-xMin / totalWidth) * width), height);			
		}
		else
		{
			totalWidth = xMin + xMax;
			topYAxis.setLocation(((xMin / totalWidth) * width), 0);
			bottomYAxis.setLocation(((xMin / totalWidth) * width), height);
		}


		if((yMin < 0) && (yMax >= 0))
		{
			totalHeight = -yMin + yMax;
			leftXAxis.setLocation(0, (-yMin / totalWidth) * height);
			rightXAxis.setLocation(width, (-yMin / totalWidth) * height);
		}
		else if((yMin < 0) && (yMax < 0))
		{
			totalHeight = -yMin - yMax;
			leftXAxis.setLocation(0, (-yMin / totalWidth) * height);
			rightXAxis.setLocation(width, (-yMin / totalWidth) * height);
		}
		else
		{
			totalHeight = yMin + yMax;
			leftXAxis.setLocation(0, (yMin / totalWidth) * height);
			rightXAxis.setLocation(width, (yMin / totalWidth) * height);
		}
	}
			
	public void paint(Graphics g)
	{
		Graphics2D g2D = (Graphics2D)g;

		g2D.setPaint(Color.white);
		Rectangle2D.Double back = new Rectangle2D.Double(0, 0, width, height);
		g2D.fill(back);
		g2D.draw(back);

		g2D.setPaint(Color.red);

		drawAxis(g2D);
		graphEq(g2D, graph);

	}

	public void resizeGraph(double xmin, double xmax, double xstep, double ymin, double ymax, double ystep, Rectangle r)
	{
		xMin = xmin;
		xMax = xmax;
		xStep = xstep;
		yMin = ymin;
		yMax = ymax;
		yStep = ystep;
		calcPoints();
		repaint();
	}

	public void updateGraph()
	{
		repaint();
	}

	class DialogListener extends WindowAdapter
	{
		public void windowClosing(WindowEvent e)
		{
		}
	}

	private void drawAxis(Graphics2D g2D)
	{
		Point2D.Double temp = new Point2D.Double();
		Line2D.Double xAxis = new Line2D.Double(leftXAxis, rightXAxis);
		Line2D.Double yAxis = new Line2D.Double(topYAxis, bottomYAxis);

		g2D.draw(xAxis);
		g2D.draw(yAxis);

		for(Point2D.Double i = leftXAxis; i.x <= rightXAxis.x; i.setLocation(i.x + (width / (totalWidth / xStep)), i.y))
		{
			temp.setLocation(i.x, i.y + 3);
			g2D.draw(new Line2D.Double(i, temp));
		}
		
		for(Point2D.Double i = topYAxis; i.y <= bottomYAxis.y; i.setLocation(i.x, i.y + (height / (totalWidth / yStep))))
		{
			temp.setLocation(i.x + 3, i.y);
			g2D.draw(new Line2D.Double(i, temp));
		}
	}

	private double calcYPart(double x)
	{
		double y = 0.0d;

		if(x < 0)
			y = (-x / totalWidth) * width;
		else
			y = (x / totalWidth) * width;

		return y;
	}

	private double calcXPart(double y)
	{
		double x = 0.0d;

		log("totalHeight is " + totalHeight);
		log("Y in calcXPart is " + y);

		if(y < 0)
			x = (-y / totalHeight) * height;
		else
			x = (y / totalHeight) * height;

		return x;
	}

	public void graphEq(Graphics2D g2D, Equation eq)
	{
		Point2D.Double temp = new Point2D.Double();
		double x, y;
		System.out.print("Graphing...");
		g2D.setPaint(Color.red);
		Line2D.Double templ = new Line2D.Double();

		drawAxis(g2D);

		for(double i = xMin; i <= xMax; i += .1)
		{
			y = eq.evaluateEquation(i);
			x = calcXPart(y);
			y = calcYPart(i);
			temp.setLocation(x, y);
			templ.
			g2D.draw(new Line2D.Double(temp, temp));
		}

		System.out.println("");
	}

	public void setEquation(Equation eq)
	{
		graph = eq;
	}

	public boolean log(String debug)
	{
		try
		{
			String dirName = "C:\\Calculus";
			String fileName = "log.graphbox.txt";

			File myDir = new File(dirName);

			if(!myDir.exists())
				myDir.mkdir();

			else if(!myDir.isDirectory())
			{
				System.err.println(dirName + " is not a directory");
				return false;
			}

			File myFile = new File(myDir, fileName);
			myFile.createNewFile();

			BufferedWriter logStream = new BufferedWriter(
							new FileWriter(myFile.getPath(), true));

			if(logNum == 0)
				logStream.write("\n" + "\n");

			logStream.write(debug + "\n");
			logStream.close();
			logNum++;
		}
		catch(IOException e)
		{
			System.err.println("IOException " + e + " occurred");
		}

		return true;
	}
}